package extensions;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.net.URL;
import java.net.Socket;
import java.net.URLDecoder;
import java.net.URLEncoder;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.ArrayList;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.zip.GZIPInputStream;
import java.util.zip.InflaterInputStream;

import dareka.Main;
import dareka.common.CloseUtil;
import dareka.common.Logger;
import dareka.common.regex.JavaMatcher;
import dareka.common.regex.JavaPattern;
import dareka.extensions.Extension;
import dareka.extensions.Rewriter;
import dareka.extensions.RequestFilter;
import dareka.extensions.Extension2;
import dareka.extensions.ExtensionManager;
import dareka.processor.HttpHeader;
import dareka.processor.HttpRequestHeader;
import dareka.processor.HttpResponseHeader;
import dareka.processor.Resource;
import dareka.processor.URLResource;
import dareka.processor.StringResource;
import dareka.processor.Processor;
import dareka.processor.TransferListener;
import dareka.processor.impl.Cache;
import dareka.processor.impl.EasyRewriter;
import dareka.processor.impl.RewriterProcessor;
import dareka.processor.impl.NicoApiUtil;

/*
 *
 */
// public class cacheCommentExtension extends URLResource implements Extension, Extension2, Processor, Rewriter, RequestFilter, TransferListener {
public class cacheCommentExtension implements Extension, Extension2, Processor, Rewriter, RequestFilter {

	// Extension interface
	public Object queryInterface(Extension.Type type) {
		try {
			if (type == Extension.Type.Extension1)
				return (Extension) this;
			if (type == Extension.Type.Processor1)
				return (Processor) new cacheCommentExtension();
			if (type == Extension.Type.RequestFilter1)
				return (RequestFilter) this;
			if (type == Extension.Type.Rewriter1)
				return (Rewriter) this;
			return null;
		} catch (IOException e) {
			Logger.debug(e);
		}
		return null;
	}

	public void registerExtensions(ExtensionManager mgr) {
		try {
			mgr.registerProcessor(new cacheCommentExtension());
			mgr.registerRewriter(this);
			mgr.registerRequestFilter(this);
		} catch (IOException e) {
			Logger.debug(e);
		}
	}

	public String getVersionString() {
		return "cacheCommentExtension (9).06a3 utf8 mod";
	}

/* コンストラクタ、初期設定 (fin) */
/***************************************************************************/
// とりあえず cache/#comment をコメントキャッシュ用にしてみる
    private static File cacheCommentDir = new File("cache/#comment");
	private static boolean alreadyInitCacheComment = false;

/*
 * コンストラクタで一度だけ初期設定をする
 */
	public cacheCommentExtension() throws IOException {
//		super("http://dummy.url.com");
		// ここにstaticなinit処理
		if (!alreadyInitCacheComment) {
Logger.debug("cc:init start");
			alreadyInitCacheComment = true;
	        // 自動コメント保存
		   	String autoCacheComment = System.getProperty("autoCacheComment", "false");
			if (autoCacheComment.equals("false"))
				Logger.info(" autoCacheComment off.");
			else if (autoCacheComment.equals("one"))
				Logger.info(" 1-comment auto cache mode.");
			else if (autoCacheComment.equals("all"))
				Logger.info(" all-comments auto cache mode.(Bad...)");
			else if (autoCacheComment.matches("[0-9]+"))
				Logger.info(" new comment auto cache mode. threshold: "+autoCacheComment+"%");
			else {
				Logger.info(" => Invalid autoCacheComment setting. autoCacheComment off...");
	            System.setProperty("autoCacheComment", "false");
			}
			if (System.getProperty("useCommentCacheNum", "true").equals("true")) {
				Logger.info(" use cacheCommentNum on.");
			} else {
				Logger.info(" use cacheCommentNum off.");
			}

			// キャッシュディレクトリ設定
			String cacheDir = System.getProperty("cacheFolder", "");
			if (!cacheDir.equals("")){ 
				cacheCommentDir = new File(cacheDir+"/#comment");
			}
	   	    id2NameMap.clear();
   		    tmp_watchId2Buf.clear();
			id2LastRes.clear();
	        cacheCommentDir.mkdir();
        	searchCacheCommentsOnADirectory(cacheCommentDir, 1);
Logger.debug("cc:init end");
		}
	}

// 引数2つのコンストラクタは、URL_POST_Resourceとして後で定義



/* Processorブロック (fin) */
/*****************************************************************************/
	// Processor Interface
    private static final String[] PROCESSOR_SUPPORTED_METHODS =
        new String[] { "GET","POST" };

	public String[] getSupportedMethods() {
	    return PROCESSOR_SUPPORTED_METHODS;
	}

    private static final Pattern PROCESSOR_SUPPORTED_PATTERN = Pattern.compile(
			"(^http://msg[0-9]*\\.nicovideo\\.jp/([0-9]+/)?api/$)|"+
			"(^http://www\\.nicovideo\\.jp/comment/.*$)|"+
			"(^http://www\\.nicovideo\\.jp/force/[a-z]{2}[0-9]+(\\?comment_cache=[a-z][0-9]+)?(?:\\?[0-9]+)?$)");

// 本体に統合
//			"(^http://[^/]+\\.nicovideo\\.jp/api/getflv(?:\\?[vms]=|/)[a-z]{2}[0-9]+(?:\\?.*)?$)");


	public Pattern getSupportedURLAsPattern() {
		return PROCESSOR_SUPPORTED_PATTERN;
	}

	public String getSupportedURLAsString() {
		return null;
	}

// 各プロセッサ用Pattern
	private static final Pattern MSG_PROCESSOR_PATTERN = Pattern.compile(
			"^http://msg[0-9]*\\.nicovideo\\.jp/([0-9]+/)?api/$");
	private static final Pattern LOCAL_PROCESSOR_PATTERN = Pattern.compile(
			"^http://www\\.nicovideo\\.jp/comment/(.*)$");
	private static final Pattern FORCE_PROCESSOR_PATTERN = Pattern.compile(
			"^http://www\\.nicovideo\\.jp/force/([a-z]{2}[0-9]+)(?:\\?comment_cache=([a-z])([0-9]+))?(?:[?&][0-9]+)?$");
//	private static final Pattern GETFLV_PROCESSOR_PATTERN = Pattern.compile(
//			"^http://[^/]+\\.nicovideo\\.jp/api/getflv(?:\\?[vms]=|/)([a-z]{2})([0-9]+)(?:\\?.*)?$");

/*
 * メッセージサーバ処理とローカルからのコメントリスト取得用
 */
	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser)
			throws IOException {
Logger.debug("cc:processor (URI):"+requestHeader.getURI());
// URIで分岐
		Resource resource = null;
// メッセージサーバへのアクセス
		Matcher m = MSG_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
		if (m.matches()) {
			resource = msgProcessorOnRequest(requestHeader, browser);
		}
// ローカルからコメントリストの取得
		m = LOCAL_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
		if (m.matches()) {
			resource = localCommentProcessorOnRequest(requestHeader, browser);
		}
// 強制再生ページの作成
		m = FORCE_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
		if (m.matches()) {
			resource = forceProcessorOnRequest(requestHeader, browser);
		}
// getflvの捏造
//		m = GETFLV_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
//		if (m.matches()) {
//			resource = getflvProcessorOnRequest(requestHeader, browser);
//		}

		if (resource == null)
			resource = new StringResource("error");
		return resource;
	}

/* CacheCommentProcessorブロック (fin) */
/* ------------------------------------------------------------------------ */
// コメントキャッシュへのアクセス用Referer
	private static final Pattern CACHECOMMENT_REFERER = Pattern
			.compile("(^(?:[^,]*,)?http://(?:www\\.nicovideo|res\\.nimg)\\.jp/(?:swf/|local/)?(?:[^/]+/)?(?:nico|flv)player(?:_wrapper)?\\.swf\\?ts=)([a-z]{2}[0-9]+)([a-z])([0-9]+)([^a-z0-9].*)?(?:,.*)?$");

// コメント自動保存用Referer
	private static final Pattern AUTOCACHE_REFERER = Pattern
			.compile("^(?:[^,]*,)?http://(?:www\\.nicovideo|res\\.nimg)\\.jp/(?:swf/(?:[^/]+/)?|local/)?(?:[^/]+/)?(?:nico|flv)player(?:_wrapper)?\\.swf\\?ts=([0-9]+t[a-z0-9]+)(?:[^a-z0-9].*)?(?:,.*)?$");

// コメント自動保存用Referer
	private static final Pattern CLICK_REVISION = Pattern
			.compile("click_revision=\"[1-9]");

// どのWatchページからのアクセスかの一意の識別子
	private String watchId = null;

/*
 * メッセージサーバ関係
 */
	private Resource msgProcessorOnRequest(HttpRequestHeader requestHeader, Socket browser)
		throws IOException {
		String referer = requestHeader.getParameter("nl-from");
		String uri = requestHeader.getURI();
Logger.debug("cc:msg processor (URI):"+uri);
Logger.debug("cc:msg processor (nl-Referer):"+referer);

		RewriterProcessor rewriter = Main.getRewriterProcessor();

// POSTの受信
		InputStream in = browser.getInputStream();
		int contentLength = (int)requestHeader.getContentLength();
		byte[] postBuf = new byte[contentLength];
		int pos = 0;

		while (pos < contentLength) {
			int len = contentLength - pos;
			pos += in.read(postBuf, pos, len);
		}
		String postString = new String(postBuf,"UTF-8");
Logger.debug("cc:msg processor (POST):"+postString);

		Matcher mAutoCache = null;
// refererがnullってことは無いはずだけど
// 別マシンで立ててて、ノー○ンさんがreferer削ってるとか
		if (referer == null) {
			referer = requestHeader.getMessageHeader("Referer");
		}
		if (referer != null) {
// 対応するRefererじゃなければそのまま
			Matcher mCache = CACHECOMMENT_REFERER.matcher(referer);
			mAutoCache = AUTOCACHE_REFERER.matcher(referer);
//			Matcher clickRevision = CLICK_REVISION.matcher(postString);
// キャッシュからのコメント取得
			if (mCache.matches()) {
// メッセージサーバにPOSTされないように
				requestHeader.setContentLength(0); 
				requestHeader.setMethod("GET");

//				if (!clickRevision.find()) {
//					return new StringResource("error");
//				}


// 投稿者コメントかどうか
// 他にもパターンあるかもしれないけど、プレミアム会員じゃないので知らん
				String flag;
				if (postString.indexOf("fork=") != -1)
					flag = AUTHOR_COMMENT;
				else
					flag = NORMAL_COMMENT;

// コメント無しモード?
				if (mCache.group(3).equals(NO_COMMENT)) {
					flag = NO_COMMENT;
				}

				String res_from = "1";
				String thread = null;
				Pattern pattern = Pattern.compile("thread=\"([0-9]+)\"");
				Matcher m = pattern.matcher(postString);
				if (m.find()) {
					thread = m.group(1);
				}
				pattern = Pattern.compile("res_from=\"([^\"]+)\"");
				m = pattern.matcher(postString);
				if (m.find()) {
					res_from = m.group(1);
				}
				long date = new Date().getTime() / 1000;

// ローカルメモリーからの自動更新のアクセスは後で処理
// 投コメは最初に読み込んだ後、自動更新はされないみたい
				if (true || postString.indexOf("res_from=\"-") != -1 || flag.equals(AUTHOR_COMMENT)) {
					String smid = mCache.group(2);
					String cc_date = mCache.group(4);
					
					// 実際にファイルを取得した場合にメッセージを表示するように変更
		//			if (postString.indexOf("res_from=\"-") != -1) Logger.info("Using Comment Cache.");

// 空のコメントを返す
					if (flag.equals(NO_COMMENT)) {
						String str =
							"<?xml version=\"1.0\" encoding=\"UTF-8\"?><packet>" +
							"<thread last_res=\"0\" resultcode=\"0\" revision=\"1\" server_time=\"" + date;
						if (thread != null) {
							str += "\" thread=\"" + thread;
						}
						str +=
							"\" ticket=\"0x0\"/>" +
							"<view_counter id=\"" + smid + "\" mylist=\"72\" video=\"765\"/>";
// 空の投コメを追加
						if (flag.equals(AUTHOR_COMMENT)) {
							str += "<thread fork=\"1\" last_res=\"0\" resultcode=\"0\" revision=\"1\" server_time=\"" + date;
							if (thread != null) {
								str += "\" thread=\"" + thread;
							}
							str += "\" ticket=\"1247525194\"/>";
						}

						str += "</packet>";
						StringResource r = new StringResource(str);
						r.addNoCacheResponseHeaders();
						return r;
					}
// 動画ID、FLAG、DATEで問い合わせ
					File file = getCommentFile(smid, NORMAL_COMMENT, cc_date);

/*
					if (file == null && flag.equals(AUTHOR_COMMENT)) {
						flag = NORMAL_COMMENT;
						file = getCommentFile(mCache.group(2), flag, mCache.group(4));
					}
*/
					if (file == null) {
						Logger.info("Cached CommentFile not found.");
						return new StringResource("error");
					}

					String content = readFile(file);
					if (content == null) {
						Logger.info("Cached CommentFile not found.");
						return new StringResource("error");
					}
// 投コメをリクエストされてて、含まれてなければ追加
					if (flag.equals(AUTHOR_COMMENT)
							&& content.indexOf("fork=") == -1) {
						String authorContent = null;
						File file2 = getCommentFile(smid, AUTHOR_COMMENT, cc_date);
						if (file2 != null) {
							authorContent = readFile(file2);
							pattern = Pattern.compile("(?s)(<thread [^>]*fork=.*</packet>)");
							m = pattern.matcher(authorContent);
							if (m.find()) {
								authorContent = m.group(1);
							} else {
								authorContent = null;
							}
						}
						if (authorContent == null) {
// 無ければ空の投コメを追加
// キャッシュからthreadを取得
							thread = null;
							pattern = Pattern.compile("thread=\"([0-9]+)\"");
							m = pattern.matcher(content);
							if (m.find()) {
								thread = m.group(1);
							}
							authorContent = "<thread fork=\"1\" last_res=\"0\" resultcode=\"0\" revision=\"1\" server_time=\"" + date;
							if (thread != null) {
								authorContent += "\" thread=\"" + thread;
							}
							authorContent += "\" ticket=\"1247525194\"/></packet>";
						}

						if (content.indexOf("</packet>") != -1) {
							content = content.replaceFirst("</packet>", authorContent);
						} else {
							Logger.info("Unable to parse CommentCache : " + smid);
						}
					}
// thread番号を修正する
/*
					if (thread != null) {
Logger.info("th:"+thread);
						content.replaceAll("thread=\"\\d+\"", "thread=\""+thread+"\"");
					}
*/
// キャッシュにもRewriterを適用
	//				StringResource r = new StringResource(rewriter.stringRewriter(uri, content));
//					HttpResponseHeader responseHeader = null;

					// 非公開公式動画だと新プレでローカルコメントを見れない対策
					// nlFilterで1000000000(架空のthreadID:Sun Sep 09 2001 10:46:40投稿に相当)を追加している
					if (postString.contains("thread=\"1000000000\"") && !content.contains("thread=\"1000000000\"")){
						content = content.replaceAll("thread=\"\\d+\"", "thread=\"1000000000\"");
					}

					StringResource r = new StringResource(rewriter.stringRewriter(uri, content, requestHeader, null));
					
					r.addNoCacheResponseHeaders();
					Logger.info("Using Comment Cache. (" + smid + " " + cc_date.replaceFirst("(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})", "$1/$2/$3 $4:$5:$6") + ")");
					return r;
				} else {
// 自動更新のリプライを捏造する
//						Logger.info("cacheCommentExtension Auto Comment-Reload Block.");
/*
					String last_res = String.valueOf(Integer.decode(res_from)-1);
					String str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"+
							"<packet><thread last_res=\""+last_res+"\" "+
							"resultcode=\"0\" revision=\"1\" "+
							"server_time=\""+date+"\"";
					if (thread != null) {
						str += " thread=\""+thread+"\"";
					}
					str += "/></packet>";
*/
					String str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"+
							"<packet></packet>";
					StringResource r = new StringResource(str);
					r.addNoCacheResponseHeaders();
					return r;
				}
			}
		}
 

// POSTの内容を置換して、コメントを投稿、取得
// Rewriterを利用して書き換える
		String postURI = uri.replaceFirst("^http://", "http://POST/");
//		postString = rewriter.stringRewriter(postURI, postString);
//		HttpResponseHeader responseHeader = null;
		postString = rewriter.stringRewriter(postURI, postString, requestHeader, null);
		postBuf = postString.getBytes("UTF-8");
		requestHeader.setMessageHeader("Content-Length", ""+postBuf.length);
		requestHeader.removeHopByHopHeaders();

//		byte[] bcontent = postServer(requestHeader, postBuf);
		URLResource ur = new URLResource(requestHeader.getURI());
		HttpResponseHeader responseHeader = ur.getResponseHeader(new ByteArrayInputStream(postBuf), requestHeader);
		byte[] bcontent = ur.getResponseBody();

		if (bcontent == null) {
			return new StringResource("error");
		}
		String content = new String(bcontent, "UTF-8");
// 自動更新じゃないなら保存処理
// autoCacheが有効じゃなければ、このRefererは来ない
/* [2011/01/03] 変更点 \\d → [1-9] wrapper(res_from="0")を対象にするため。
//(wrapperの自動更新も対象になってるはず。だけど、res_from="0"だから問題ないよね。)
		if ( (!postString.matches(".*res_from=\"\\d.*")
					|| postString.contains("fork="))
					&& mAutoCache != null
					&& mAutoCache.matches()) {
*/
/*
Logger.info("[debug] cc:watchId = "+ watchId + ":" +content.length()+"");
Pattern pattern = Pattern.compile("</chat>");
Matcher m = pattern.matcher(content);
int commentLen = 0;
while (m.find()) {
	commentLen++;
}
Logger.info("[debug] cc:  (" + commentLen + "件)");
*/
		if ( postString.contains("<thread") && content.contains("</chat>")
				 && (!postString.matches(".*res_from=\"[1-9].*")
					|| postString.contains("fork="))
					&& mAutoCache != null
					&& mAutoCache.matches()) {
			watchId = mAutoCache.group(1);
			if (!System.getProperty("autoCacheComment", "false").equals("false")){
				storeComment(bcontent, watchId, postString);
			}
		}
		else {
			Pattern id_re = Pattern.compile("\"\\s+id=\"([a-z0-9]+)\"\\s+");
			Matcher m = id_re.matcher(content);
			if (m.find()) {
				String video_id = m.group(1);
Logger.debug("cc:msg processor (video_id):"+video_id);
				if (!System.getProperty("autoCacheComment", "false").equals("false")){
					storeComment(bcontent, video_id, postString);
				}
			}
		}

// Rewriterを呼び出す
		StringResource r;
		if (postString.contains("postkey=\"")) {
			r = new StringResource(content);
		} else {
//			r = new StringResource(rewriter.stringRewriter(uri, content));
			r = new StringResource(rewriter.stringRewriter(uri, content, requestHeader, responseHeader));
		}
		r.addResponseHeader(HttpHeader.CONTENT_TYPE, "text/xml");
		r.addNoCacheResponseHeaders();
		return r;
	}

	private String readFile(File file) throws IOException {
		BufferedInputStream in = new BufferedInputStream(new FileInputStream(file));
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		int len;
		byte[] buf = new byte[256];
		try {
			while ( (len = in.read(buf, 0, 256)) != -1)
				out.write(buf, 0, len);
		} catch (IOException e) {
			return null;
		} finally {
			CloseUtil.close(in);
			CloseUtil.close(out);
		}
		return new String(out.toByteArray(), "UTF-8");
	}



/* CommentDirProcessorブロック (fin) */
/* ------------------------------------------------------------------------ */

/*
 * ローカルからコメントリストの取得、ブラウザのフォームからコメント保存など
 */
	private Resource localCommentProcessorOnRequest(
				HttpRequestHeader requestHeader, Socket browser)
				throws IOException {
Logger.debug("cc:local processor (URI):"+requestHeader.getURI());
		String path = "";
		Matcher m = LOCAL_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
		if (m.matches()) {
			path = m.group(1);
		}

// comment/cacheCommentList.json?[動画ID] でその動画のコメントのリスト
		if (path.startsWith("cacheCommentList.json?")) {
			String id = path.substring(22);
			String str = getCacheCommentDateAsJson(id);
			if (str == null) {
				return new StringResource("");
			}
			StringResource r = new StringResource(str);
			r.addNoCacheResponseHeaders();
			r.addResponseHeader(HttpHeader.CONTENT_TYPE, "application/x-javascript");
			return r;
		}

// comment/cacheCommentSave?[動画ID]&[取得コメント数]
// 動画IDの現在のコメントを保存
// &以降は省略可能、省略時はニコニコのデフォルト数
		else if (path.startsWith("cacheCommentSave?")) {
			String parm = path.substring(17);
			if (saveComment(requestHeader, parm)) {
				StringResource r = new StringResource("OK");
				r.addNoCacheResponseHeaders();
				return r;
			}
			return new StringResource("NG");
		}

// comment/removeCacheComment?[動画ID](_[DATE])
// コメント削除
		else if (path.startsWith("removeCacheComment?")) {
			String parm = path.substring(19);
			Pattern pattern = Pattern.compile("([a-z]{2}[0-9]+)(?:_([0-9]+))");
			m = pattern.matcher(parm);
			if (m.matches()) {
				removeCommentFile(m.group(1), m.group(2));
			} else {
				removeCommentFile(parm);
			}
			StringResource r = new StringResource("OK");
			r.addNoCacheResponseHeaders();
			return r;
		}

// comment/removeCommentExistMovie
// 動画のないコメント削除
		else if (path.equals("removeCommentExistMovie")) {
			removeCommentExistMovie();
			StringResource r = new StringResource("OK");
			r.addNoCacheResponseHeaders();
			return r;
		}

// comment/removeShortComment?
// 引数未満のコメント数のキャッシュを削除
		else if (path.startsWith("removeShortComment?")) {
			int commentNum = Integer.parseInt(path.substring(19));
			removeShortCommentFile(commentNum);
			StringResource r = new StringResource("OK");
			r.addNoCacheResponseHeaders();
			return r;
		}

// コメントキャッシュ管理ページ
		else if (path.equals("")) {
			String content =
// ヘッダ
			"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n" +
			"<html>\n<head>\n" +
			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n" +
			"<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">\n" +
			"<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n" +
			"<meta http-equiv=\"Pragma\" content=\"no-cache\">\n" +
			"<meta http-equiv=\"Cache-Control\" content=\"no-store, no-cache, must-revalidate, post-check=0, pre-check=0\">\n" +
			"<meta http-equiv=\"Expires\" content=\"Thu, 01 Dec 1994 16:00:00 GMT\">\n" +
			"<script type=\"text/javascript\" src=\"http://res.nimg.jp/js/lib/prototype.js?1.5.1.1_2\"></script>\n" +

			"<title>コメントキャッシュ管理ページ</title>\n" +
			"<base href=\"http://www.nicovideo.jp/\">\n" +
			"</head>\n" +

			"<body>\n" +
// ページヘッダ
			"<div id=\"PAGEHEADER\">\n" +
			"<div align=\"right\">\n" +
			"<table border=\"0\" cellpadding=\"0\" cellspacing=\"4\" style=\"font-size:12px;\">\n" +
 			"<tr><td><a href=\"\">トップページ</a> | <a href=\"my\">マイページ</a> | "+
			"<a href=\"history\">最近見た動画</a> | <a href=\"newarrival\">新着投稿動画</a> | "+
			"<span id=\"nlbar\"><a href=\"/cache/\">キャッシュ</a>"+
			"</td></tr></table>" +
			"</div>\n" +
			"</div>\n" +

//ページ本体
			"<div id=\"PAGEBODY\">\n" +
			"<h1>コメントキャッシュ管理ページ</h1>\n" +
// 保存したコメントリストの表示用
// スクリプトファイル読み込み
			"<script type=\"text/javascript\" charset=\"utf-8\" src=\"" + addTS("/local/cacheComment.js") + "\"></script>\n"+
// コメント管理用フォーム
			"<form action=\"#\">\n"+
			"<input id=\"removeCommentExistMovieButton\" type=\"button\" style=\"padding-left:0;\"value=\"動画キャッシュのないコメントキャッシュの削除\" onclick=\"removeCommentExistMovie()\" class=\"submit\">\n"+
			"</form><p>\n"+

			"<form action=\"#\">\n"+
			"<input id=\"removeShortCommentLimit\" type=\"text\" name=\"limit\" size=\"5\" value=\"50\">\n"+
			"<input id=\"removeShortCommentButton\" type=\"button\" value=\"指定数未満のコメント数のキャッシュを削除\" onclick=\"removeShortComment()\" class=\"submit\">\n"+
			"</form><br>\n"+
			"※100以上にすると1分未満の動画のコメントがすべて消えるので注意\n"+

			"</div>\n" +
			"</body></html>\n";

			StringResource r = new StringResource(content);
			r.addNoCacheResponseHeaders();
			return r;
		}

		return null;
	}

/* ForceProcessorブロック (fin) */
/* ------------------------------------------------------------------------ */

/*
 * 強制再生用のページを作成する
 */

// 強制再生ページ用のプレイヤーバージョンとか
	private static String playerVersion = null;
	private static final String PLAYER_VER_DEFAULT = "1286521293";
//	private static String playerVersionXml = null;
//	private static String marqueeVersion = null;


	private Resource forceProcessorOnRequest(
				HttpRequestHeader requestHeader, Socket browser)
				throws IOException {
Logger.debug("cc:force processor (URI):"+requestHeader.getURI());
		String smid, commentId, flag, tsFlag;
		Matcher m = FORCE_PROCESSOR_PATTERN.matcher(requestHeader.getURI());
		if (m.matches()) {
			smid = m.group(1);
			flag = m.group(2);
			commentId = m.group(3);
		} else {
			return new StringResource("error");
		}
		
		/*	----2010/10/12変更点---ここから----
		String threadId = smid;
		if (smid.substring(0,2).equals("so")) {
			if (m.matches()){
				File file = getCommentFile(smid, flag, commentId);
				if (file != null) {
					String tmp_Comment = readFile(file);
					m = Pattern.compile("thread=\"([0-9]+)\"").matcher(tmp_Comment);
					if (m.find()) {
						threadId = m.group(1);
					}
				}
			} else {
				
			}
		}
		//	----2010/10/12変更点---ここまで----/*/
		
// パラメータが無ければコメントは取りに行ってみる
		if (flag == null) {
			if (smid.indexOf("so") == 0) {
				flag = "a";
				commentId = "0";
				tsFlag = smid + flag + commentId;
			} else {
				tsFlag = getWatchIdComment() + smid;
				flag = "0";
				commentId = "-1";
			}
		} else {
			tsFlag = smid + flag + commentId;
		}
		
		String ver;
		if (playerVersion == null) {
			Logger.info("playerVersion is not set.Use Default.");
			ver = PLAYER_VER_DEFAULT;
		} else {
			ver = playerVersion;
		}

		Cache cache = new Cache(smid);
		if (!cache.exists()) {
			return new StringResource("error");
		}
		String title = cache.getTitle();
		String extension = cache.getPostfix();
		
		String swfStr = System.getProperty("forcePagePlayerURL", "http://res.nimg.jp/swf/player/nicoplayer.swf");

		String content;
		content =
// ヘッダ
			"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n" +
			"<html>\n<head>\n" +
			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n" +
			"<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">\n" +
			"<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n" +
			"<meta http-equiv=\"Pragma\" content=\"no-cache\">\n" +
			"<meta http-equiv=\"Cache-Control\" content=\"no-store, no-cache, must-revalidate, post-check=0, pre-check=0\">\n" +
			"<meta http-equiv=\"Expires\" content=\"Thu, 01 Dec 1994 16:00:00 GMT\">\n" +
			"<title>" + title +"</title>\n" +
			"<base href=\"http://www.nicovideo.jp/\">\n" +
			addTS2(System.getProperty("forcePageHeader1", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader2", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader3", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader4", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader5", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader6", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader7", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader8", "")) + "\n" +
			addTS2(System.getProperty("forcePageHeader9", "")) + "\n" +
			"</head>\n" +

			"<body>\n" +
// ページヘッダ
			"<div id=\"PAGEHEADER\">\n" +
			"<div align=\"right\">\n" +
			"<table border=\"0\" cellpadding=\"0\" cellspacing=\"4\" style=\"font-size:12px;\">\n" +
 			"<tr><td><a href=\"\">トップページ</a> | <a href=\"my\">マイページ</a> | <a href=\"history\">最近見た動画</a> | " +
			"<span id=\"nlbar\"><a href=\"/cache/\">キャッシュ</a> (<a href=\"/cache/" +
			smid + "/movie\">保存</a>・" +
			"<a href=\"/cache/" + smid + "/audio\">音声</a>・" +
 			"<a href=\"/cache/rm?" + smid +"\">削除</a>)</span> | <a href=\"https://secure.nicovideo.jp/secure/logout\">ログアウト</a>"+
			"</td></tr></table>" +
			"</div>\n" +
			"</div>\n" +

//ページ本体
			"<div id=\"PAGEBODY\">\n" +
			"<div id=\"WATCHHEADER\">\n" +
			"<h1>" + title + "</h1>\n" +
			"<table border=\"0\" cellspacing=\"4\" cellpadding=\"0\" summary=\"コメント\"><tr>\n" +
			"<td style=\"color:orange\">";
		if (flag.equals("0")) {
			content += "最新コメントを取得再生";
		} else if (flag.equals(NO_COMMENT)) {
			content += "コメント表示無しで再生";
			commentId = "0";
		} else {
			content += " 20" +
			commentId.substring(0,2)+"年"+
			commentId.substring(2,4)+"月"+
			commentId.substring(4,6)+"日 "+
			commentId.substring(6,8)+"時"+
			commentId.substring(8,10)+"分"+
			commentId.substring(10)+"秒 時点のローカルメモリー";
		}
		content += "</td>\n" +
// 削除ボタン
			"<td><form action=\"#\">\n"+
			"<input id=\"removeCacheCommentButton\" type=\"button\" value=\"コメ削除\" onclick=\"removeCacheComment()\" class=\"submit\">\n"+
			"<input id=\"removeAllCacheCommentButton\" type=\"button\" value=\"この動画の全コメを削除\" onclick=\"removeAllCacheComment()\" class=\"submit\">\n"+
			"</form></td>\n"+

// 保存したコメントリストの表示用
			"<td id=\"cacheCommentDropList\"></td>\n" +
// コメント管理用
			"<td><a href=\"http://www.nicovideo.jp/comment/\">コメ管理</a></td>\n"+
			"</tr></table>\n" +
// スクリプトファイル読み込み
			"<script type=\"text/javascript\" charset=\"utf-8\" src=\"" + addTS("/local/cacheComment.js") + "\"></script>\n"+
// コメントキャッシュリスト読み込み用動画ID&読み込みスタート
			"<script type=\"text/javascript\">\n"+
			"cacheCommentId=\"" + smid + "\";\n"+
			"cacheCommentPrefix=\"" + smid + "\";\n"+
				// cacheCommentDateはNumber型に。
			"cacheCommentDate="+ commentId + ";\n"+
			"startCacheComment(\"force\");\n"+
			"</script>\n"+
			"</div>\n" +

// プレイヤー部分
			"<div id=\"flvplayer_container\" style=\"width:984px; height:540px; padding:4px; clear:both; overflow:hidden;\">\n" +
			"<h3 style=\"color:#F30; text-align:center;\">\n" +
			"<script type=\"text/javascript\"><!--\n" +
			"document.write('最新の <a href=\"http://www.adobe.com/go/getflashplayer\" style=\"color:#F30;\">Adobe Flash Player</a> にアップデートしてください');\n" +
			"--></script>\n" +
			"<noscript>JavaScript が無効になっているためプレイヤーを表示できません</noscript>\n" +
			"</h3></div>\n" +
			"<script type=\"text/javascript\"><!--\n" +
// 再生用tsの設定
			"var so = new SWFObject(\"" + swfStr +
			"?ts=" + tsFlag + "\", \"flvplayer\", \"984px\", \"540px\", 10, \"#FFFFFF\");\n" +
/*
			"	so.addVariable(\"v\",  \"" + smid + "\");\n" +
			"	so.addVariable(\"videoId\",  \"" + smid + "\");\n" +
			"	so.addVariable(\"deleted\", \"0\");\n" +
			"	so.addVariable(\"thumbTitle\", \"" + URLEncoder.encode(title,"utf-8") + "\");\n" +
			"	so.addVariable(\"thumbDescription\", \" \");\n" +
	
//			"	so.addVariable(\"mylist_counter\", \"0\");\n" +
			"	so.addVariable(\"movie_type\", \"" + extension.replace(".", "") +"\");\n" +
			"	so.addVariable(\"thumbImage\", \"http://tn-skr1.smilevideo.jp/smile?i=" + smid.substring(2) + "\");\n" +
//			"	so.addVariable(\"userSex\", \"0\");\n" +
//			"	so.addVariable(\"userAge\", \"20\");\n" +
//			"	so.addVariable(\"us\", \"0\");\n" +
//			"	so.addVariable(\"wv_id\", \"" + smid + "\");\n" +
//			"	so.addVariable(\"wv_title\", \"test\");\n" +
//			"	so.addVariable(\"wv_code\", \"464b375c\");\n" +
//			"	so.addVariable(\"wv_time\", \"1224240789\");\n" +
			"	so.addVariable(\"player_version_xml\", \"" + ver + "\");\n" +
			"	so.addVariable(\"button_threshold\", \"0\");\n";
*/
//			"	so.addVariable(\"community_id\", \"502629\");\n" +
//			"	so.addVariable(\"community_global_id\", \"ch640\");\n" +
			"	so.addVariable(\"button_threshold\", \"0\");\n" +
			"	so.addVariable(\"thumbTitle\", \"" + URLEncoder.encode(title,"utf-8") + "\");\n" +
			"	so.addVariable(\"thumbDescription\", \" \");\n" +
			"	so.addVariable(\"videoTitle\", \" \");\n" +
			"	so.addVariable(\"threadPublic\", \"1\");\n" +
//			"	so.addVariable(\"is_channel\", \"1\");\n" +
			"	so.addVariable(\"player_version_xml\", \"" + ver + "\");\n" +
			"	so.addVariable(\"player_info_xml\", \"1291871766\");\n" +
			"	so.addVariable(\"marqueeVersion\", \"1294802479\");\n" +
//			"	so.addVariable(\"userPrefecture\", \"48\");\n" +
//			"	so.addVariable(\"csrfToken\", \"1-1295675288-3d26f4925e21df572ef8f78e8a94373bc3a7c894\");\n" +
			"	so.addVariable(\"v\", \"" + smid + "\");\n" +
			"	so.addVariable(\"videoId\", \"" + smid + "\");\n" +
			"	so.addVariable(\"deleted\", \"0\");\n" +
//			"	so.addVariable(\"mylist_counter\", \"0\");\n" +
//			"	so.addVariable(\"mylistcomment_counter\", \"0\");\n" +
			"	so.addVariable(\"movie_type\", \"" + extension.replace(".", "") +"\");\n" +
			"	so.addVariable(\"thumbImage\", \"http%3A%2F%2Ftn-skr1.smilevideo.jp%2Fsmile%3Fi%3D" + smid.substring(2) + "\");\n" +
//			"	so.addVariable(\"videoUserId\", \"0\");\n" +
//			"	so.addVariable(\"userSex\", \"0\");\n" +
//			"	so.addVariable(\"userAge\", \"1\");\n" +
//			"	so.addVariable(\"us\", \"0\");\n" +
//			"	so.addVariable(\"ad\", \"web_pc_player_marquee\");\n" +
//			"	so.addVariable(\"iee\", \"1\");\n" +
//			"	so.addVariable(\"is_community_video\", \"1\");\n" +
//			"	so.addVariable(\"is_community_thread\", \"1\");\n" +
//			"	so.addVariable(\"channelId\", \"ch640\");\n" +
//			"	so.addVariable(\"channelTopURL\", \"http%3A%2F%2Fch.nicovideo.jp%2Fchannel%2Fch640%3Fplayer_icon\");\n" +
//			"	so.addVariable(\"channelVideoURL\", \"http%3A%2F%2Fch.nicovideo.jp%2Fvideo%2Fch640%3Fplayer_icon\");\n" +
//			"	so.addVariable(\"channelName\", \" \");\n" +
//			"	so.addVariable(\"communityPostURL\", \"http%3A%2F%2Fcom.nicovideo.jp%2Fpost%2Fsm13350648%3Fplayer_icon\");\n" +
			"	so.addVariable(\"dicArticleURL\", \"http%3A%2F%2Fdic.nicovideo.jp%2Fv%2F" + smid + "\");\n" +
			"	so.addVariable(\"blogEmbedURL\", \"%2Fembed%2F" + smid + "%3Fplayer_icon\");\n" +
			"	so.addVariable(\"uadAdvertiseURL\", \"http%3A%2F%2Fuad.nicovideo.jp%2Fads%2F%3Fvid%3D" + smid + "%26player_icon1\");\n" +
//			"	so.addVariable(\"useChecklistCache\", \"1\");\n" +
			"	so.addVariable(\"noBanner\", \"1\");\n" +
//			"	so.addVariable(\"category\", \"%E8%B8%8A%E3%81%A3%E3%81%A6%E3%81%BF%E3%81%9F\");\n" +
//			"	so.addVariable(\"categoryGroupKey\", \"g_try\");\n" +
//			"	so.addVariable(\"categoryGroup\", \"%E3%82%84%E3%81%A3%E3%81%A6%E3%81%BF%E3%81%9F\");\n" +
			"	so.addVariable(\"isWide\", \"1\");\n" +
			"	so.addVariable(\"wv_id\", \"" + smid + "\");\n" +
			"	so.addVariable(\"wv_title\", \"" + URLEncoder.encode(title,"utf-8") + "\");\n" +
//			"	so.addVariable(\"wv_code\", \"c7f61d1f\");\n" +
//			"	so.addVariable(\"wv_time\", \"1295588888\");\n" +
//			"	so.addVariable(\"leaf_switch\", \"0\");\n" + 
//			"	so.addVariable(\"isOfficialAnime\", \"1\");\n" +
			"	so.addVariable(\"isNicoPlayer4\", \"1\");\n" +
			"";
// マイメモリーモードにするかどうか
		if (!flag.equals("0")) {
			content += "	so.addVariable(\"mm\", \"1\");\n";
		}

// 投稿者コメントがあるかどうか
/*
		if (flag.equals(AUTHOR_COMMENT)) {
			content += "	so.addVariable(\"has_owner_thread\", \"1\");\n";
		}
*/
		content +=
			"	so.addParam(\"allowScriptAccess\", \"always\");\n" +
			"	so.addParam(\"allowFullScreen\", \"true\");\n" +
			"	so.write(\"flvplayer_container\");\n" +
			"--></script>\n" +
			"<div id=\"WATCHFOOTER\"></div>\n" +
			"</div>\n" +
			"<div id=\"PAGEFOOTER\"><br><br><br><br><br></div>\n" +
			"</body></html>\n";



		StringResource r = new StringResource(content);
		r.addNoCacheResponseHeaders();
		return r;
	}


/* ReWriterブロック (fin) */
/****************************************************************************/
	// Rewriter interface
    private static final Pattern WATCH_PAGE_PATTERN
    	= Pattern.compile("^http://www\\.nicovideo\\.jp/watch/([a-z]{2}[0-9]+|\\d+)(\\?.*)?$");
	
	public Pattern getRewriterSupportedURLAsPattern() {
		return WATCH_PAGE_PATTERN;
	}

	// ID取得用
    private static final Pattern VIDEO_ID_PATTERN
    	= Pattern.compile("\"videoId\", \"([a-z]{2}\\d+)\"");

/*
 * watchページ用Rewriter
 */
	public String onMatch(Matcher match, HttpResponseHeader responseHeader, String content)
			throws IOException {
		JavaPattern[] p = EasyRewriter.getMatch("cacheCommentConf");
		String[] r = EasyRewriter.getReplace("cacheCommentConf");

		String smid = match.group(1);
		// [nl] マイメモリの場合は内容からIDを取ってくる (#026)
		if (smid.matches("\\d+")) {
			Matcher m = VIDEO_ID_PATTERN.matcher(content);
			if (m.find()) {
				smid = m.group(1);
			}
// マイメモリでなければ、削除・非公開チェック
		} else {
			JavaMatcher jm;
			if (p != null) {
				jm = p[0].matcher(content); 
			} else {
				jm = JavaPattern.compile("(?s)" +
					"<p class=\"[^\"]*\">(この動画は非表示に設定されています。|動画 <strong>[^<]*</strong> は再生できません。現在アクセスできないか、削除された可能性があります。).*?</div>|<span id=\"deleted_message_default\">[^<]*</span>.*?</div>"
					).matcher(content);
			}
			
			if (jm.find() && content.indexOf("id=\"flvplayer_container\"") == -1) {
				Cache cache = new Cache(smid);
				if (cache.exists()) {
					content = jm.replaceFirst(
							"$0"+
							"<div><p><h1 style=\"color:red\">強制再生</h1><span id=\"cacheCommentDropList\"></span></p>\n"+
// スクリプト読み込み&コメントキャッシュリスト読み込み
							"<script type=\"text/javascript\" charset=\"utf-8\" src=\"" + addTS("/local/cacheComment.js") + "\"></script>\n"+
							"<script type=\"text/javascript\">\n"+
							"cacheCommentId=\""+ smid +	"\";\n"+
							"cacheCommentPrefix=\"" + smid +"\";\n"+
							"startCacheComment(\"force\");\n"+
							"</script>\n"
						);
					return content;
				}
			}
		}

// チャンネルだとマイメモリみたいなアドレスなのでページ内容から判定
		String pageId = match.group(1);
		String cacheCommentPrefix = match.group(1);
		if (content.indexOf("(\"mm\", \"1\")") == -1) {
			pageId = smid;
		} else {
			cacheCommentPrefix = smid;
		}

// 元のurlがあればそちら、無ければ今のurlを使う
		String url = responseHeader.getParameter("nl-to");
		if (url == null) {
			url = match.group(0);
		}

		Pattern pattern = Pattern.compile("http://www\\.nicovideo\\.jp/watch/([a-z]{2}[0-9]+|\\d+)(?:\\?.+)?\\?comment_cache=([a-z])([0-9]+)(?:[\\?&].+)?");
		boolean useCache = true;
		Matcher urlMatch;
		String flag = "";
		String date = "";

		urlMatch = pattern.matcher(url);
		if (!urlMatch.matches()) {
			useCache = false;
			pattern = Pattern.compile("http://www\\.nicovideo\\.jp/watch/([a-z]{2}[0-9]+|\\d+)(?:\\?.+)?");
			urlMatch = pattern.matcher(url);
			if (!urlMatch.matches()) {
				Logger.warning("cacheComment:watch-pattern error.");
				return content;
			}
		} else {
			flag = urlMatch.group(2);
			date = urlMatch.group(3);
		}

// プレイヤーのバージョン文字列を取得する
		pattern = Pattern.compile("\"player_version_xml\"\\s*,\\s*\"(\\d+)\"");
		Matcher m = pattern.matcher(content);
		if (m.find()) {
			playerVersion = m.group(1);
		}

// 必要があれば、playerのts=以下の部分を変更する
		pattern = Pattern.compile("(new SWFObject\\(\"(?:http://(?:www\\.nicovideo|res\\.nimg)\\.jp/)?(?:swf/|local/)?(?:[^/]+/)?(?:nico|flv)player(?:_wrapper)?\\.swf\\?ts=)([0-9]+)\"");
		m = pattern.matcher(content);
		if (m.find()) {
// 書き換え前のtsの値を保存する
			tsParameter = m.group(2);
			if (useCache) {

// ローカルメモリー再生ページの書き換え--------------------
// ts= にID+flag+dateを設定する
// 保存ファイル名はID+date+flagの順(ソートするため)だけど、
// ここではIDとdateのセパレータとしてflagを入れているのでこの順
				content = m.replaceFirst("$1"+smid+flag+date+"\"");

// マイメモリーモードにする
				content = content.replaceFirst(
					"so\\.addVariable\\(\"v\"",
					"so.addVariable(\"mm\", \"1\");\n\t$0");
				content = content.replaceFirst(
					"(isMymemory:\\s+)false,",
					"$1true,");

// パラメータのフラグから、投稿者コメントがあるかでflvplayerのモードを変える
/*
				pattern = Pattern.compile("so\\.addVariable\\(\"has_owner_thread\", \"1\"\\);");
				m = pattern.matcher(content);
				if (m.find()
						&& urlMatch.group(2).equals(NORMAL_COMMENT)) {
					content = m.replaceFirst("");
				} else if (!m.find()
						&& urlMatch.group(2).equals(AUTHOR_COMMENT)) {
					pattern = Pattern.compile("so\\.addVariable\\(\"v\"");
					m = pattern.matcher(content);
					if (m.find()) {
						content = m.replaceFirst("so.addVariable(\"has_owner_thread\", \"1\");$0");
					}
				}
*/

// 置換パターン読み込み
				JavaMatcher jm;
				String repString;
				if (p != null) {
					jm = p[2].matcher(content);
					repString = r[2];
				} else {
					jm = JavaPattern.compile(
						"(?s)<td><form action=\"/mymemory_register\".*?</td>\\s*<td>.*?value=\"コメント編集\".*?</td>"
						).matcher(content);
					repString = "<td><ccMenu></td>";
				}

				if (jm.find()) {
// 日付とかボタンとか
					String repStr = repString.replaceFirst("<ccMenu>",
						"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">" +
/*						"<tr><td><span class=\"font12\" style=\"color:orange\"> 20"+
						urlMatch.group(3).substring(0,2)+"年"+
						urlMatch.group(3).substring(2,4)+"月"+
						urlMatch.group(3).substring(4,6)+"日 "+
						urlMatch.group(3).substring(6,8)+"時"+
						urlMatch.group(3).substring(8,10)+"分"+
						urlMatch.group(3).substring(10)+"秒 時点のローカルメモリー</span></td>"+
*/				
						"<tr><td><span class=\"font12\" style=\"color:orange\"> "+
						"20"+
						date.substring(0,2)+"/"+
						date.substring(2,4)+"/"+
						date.substring(4,6)+" "+
						date.substring(6,8)+":"+
						date.substring(8,10)+"&nbsp;</span></td>"+
// 削除ボタン
						"<td><form action=\"#\">"+
						"<input id=\"removeCacheCommentButton\" type=\"button\" value=\"コメ削除\" title=\"再生中のコメント1件を削除\" onclick=\"removeCacheComment()\" class=\"submit\">"+
						"<input id=\"removeAllCacheCommentButton\" type=\"button\" value=\"全削除\" title=\"この動画の全コメントを削除\" onclick=\"removeAllCacheComment()\" class=\"submit\">"+
						"</form></td>"+
// 保存したコメントリストの表示用
						"<td id=\"cacheCommentDropList\"></td>" +
// コメント管理用
						"<td><a href=\"http://www.nicovideo.jp/comment/\">管</a></td>"+
						"</tr></table>"+
// スクリプトファイル読み込み
						"<script type=\"text/javascript\" charset=\"utf-8\" src=\"" + addTS("/local/cacheComment.js") + "\"></script>"+
// コメントキャッシュリスト読み込み用動画ID&読み込みスタート
						"<script type=\"text/javascript\">"+
						"cacheCommentId=\""+smid+"\";\n"+
						"cacheCommentPrefix=\"" + cacheCommentPrefix +"\";\n"+
						"cacheCommentDate=" + date + ";\n"+
						"startCacheComment(\"cache\");\n"+
						"</script>\n"
					);

					content = jm.replaceFirst(repStr);
				}

// ローカルメモリー再生ページここまで---------------------------------
			} else {
// 通常のwatchページへの埋め込み -------------------------------------

// autoCacheComment用watchId埋め込み
// 一意のIDを埋め込む
				if (!System.getProperty("autoCacheComment", "false").equals("false")) {
					content = m.replaceFirst
						("$1"+getWatchIdComment()+pageId+"\"");
				}

// 置換パターン読み込み
				JavaMatcher jm;
				String repString;
				if (p != null) {
					jm = p[1].matcher(content);
					repString = r[1];
				} else {
					jm = JavaPattern.compile(
						"(?s)<td><form action=\"/mymemory_register\".*?</td>\\s*<td>.*?value=\"コメント編集\".*?</td>"
						).matcher(content);
					repString = "$0<td><ccMenu></td>";
				}

// コメント保存ボタンとか
				if (jm.find()) {
					String repStr = repString.replaceFirst("<ccMenu>",
						"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">" +
						"<tr>" +
						"<td>"+
// コメント保存ボタン
						"<form action=\"#\">"+
						"<input id=\"saveCacheCommentButton\" type=\"button\" value=\"保存(Local)\" onclick=\"saveCacheComment()\" class=\"submit\">"+
						"</form>"+
						"</td>"+

// 保存したコメントリストの表示用
						"<td id=\"cacheCommentDropList\"></td>"+
// コメント管理用
						"<td><a href=\"http://www.nicovideo.jp/comment/\">管</a></td>"+
						"</tr></table>\n"+
// スクリプト読み込み&コメントキャッシュリスト読み込み
						"<script type=\"text/javascript\" charset=\"utf-8\" src=\"" + addTS("/local/cacheComment.js") + "\"></script>\n"+
						"<script type=\"text/javascript\">\n"+
						"cacheCommentId=\""+smid+"\";\n"+
						"cacheCommentPrefix=\"" + cacheCommentPrefix +"\";\n"+
						"startCacheComment(\"watch\");\n"+
						"</script>\n"
					);
					content = jm.replaceFirst(repStr);
				}
// 通常のwatchページへの埋め込み ここまで -----------------------------
			}
		}

		return content;
	}

/* RequestFilterブロック (fin?) */
/****************************************************************************/
// 書き換え対象のパターン
	private static final Pattern CACHECOMMENT_PATTERN1 = Pattern
			.compile("http://.*(?:nicovideo|niconicommons)\\.jp(?!/force/).*(?:\\?|%3F)comment_cache(?:=|%3D)[a-z][0-9]+(?:[^0-9].*)?");
//			.compile("http://.*(?:\\?|%3F)comment_cache(?:=|%3D)[a-z][0-9]+(?:[^0-9].*)?");

	private static final Pattern CACHECOMMENT_PATTERN2 = Pattern
			.compile("(^(?:[^,]*,)?http://(?:www\\.nicovideo|res\\.nimg)\\.jp/(?:swf/|local/)?(?:[^/]+/)?(?:nico|flv)player(?:_wrapper)?\\.swf\\?ts=)([a-z]{2}[0-9]+[a-z][0-9]+)([^a-z0-9].*)?(?:,.*)?$");

	private static final Pattern FORCEPLAY_PATTERN = Pattern
			.compile("^http://www\\.nicovideo\\.jp/force/([a-z]{2}[0-9]+)(\\?comment_cache=[a-z][0-9]+)?$");

	private static final Pattern AUTOCACHE_PATTERN = Pattern
			.compile("(^(?:[^,]*,)?http://(?:www\\.nicovideo|res\\.nimg)\\.jp/(?:swf/|local/)?(?:[^/]+/)?(?:nico|flv)player(?:_wrapper)?\\.swf\\?ts=)([0-9]+t[a-z0-9]+)([^a-z0-9].*)?(?:,.*)?$");

// 書き換え前のtsを保存。万が一無かったらデフォルトを使用
	private static String tsParameter = null;
	private static final String TS_DEFAULT = "1259739382";

/*
 * コメントキャッシュ用に書き換えたURLとRefererを戻して、
 * ヘッダの内部処理用ワークにストアする
 */
	public int onRequest(HttpRequestHeader requestHeader) 
    			throws IOException {
// Refererの処理
		String referer = requestHeader.getMessageHeader("Referer");
		if (referer != null) {
			String from = null;
// 強制再生ページからのReferer
			Matcher m = FORCEPLAY_PATTERN.matcher(referer);
			if (m.matches()) {
				referer = "http://www.nicovideo.jp/watch/"+m.group(1);
				from = m.group(0);
			}
// ローカルメモリーからのReferer
			m = CACHECOMMENT_PATTERN1.matcher(referer);
			if (m.matches()) {
				referer = referer.replaceAll("(?:\\?|%3F)comment_cache(?:=|%3D)[a-z][0-9]+", "");
				referer = referer.replaceFirst("([^\\?]+)&", "$1?");
				from = m.group(0);
			}
// ローカルメモリーのflvplayerからのReferer
			m = CACHECOMMENT_PATTERN2.matcher(referer);
			if (m.matches()) {
				String tail = m.group(3);
				if (tail == null)
					tail ="";
				if (tsParameter == null) {
					Logger.info("ts is not set. Use default.");
					referer = m.group(1)+TS_DEFAULT+tail;
				} else {
					referer = m.group(1)+tsParameter+tail;
				}
				from = m.group(0);
			}
// コメント自動保存用IDのついたreferer
			m = AUTOCACHE_PATTERN.matcher(referer);
			if (m.matches()) {
				String tail = m.group(3);
				if (tail == null)
					tail ="";
				if (tsParameter == null) {
					Logger.info("ts is not set. Use default.");
					referer = m.group(1)+TS_DEFAULT+tail;
				} else {
					referer = m.group(1)+tsParameter+tail;
				}
				from = m.group(0);
			}
// Refererを書き戻す
			requestHeader.setMessageHeader("Referer", referer);
			if (from != null)
				requestHeader.setParameter("nl-from", from);
		}
// URIの処理
		String uri = requestHeader.getURI();
		if (uri != null) {
			String to = null;
// ローカルメモリーへのアクセス
			Matcher m = CACHECOMMENT_PATTERN1.matcher(uri);
			if (m.matches()) {
				uri = uri.replaceAll("(?:\\?|%3F)comment_cache(?:=|%3D)[a-z][0-9]+", "");
				uri = uri.replaceFirst("([^\\?]+)&", "$1?");
				to = m.group(0);
// ローカルメモリーにアクセスする時は、refererをマイリストに
// ブックマークからなら放置
/*
				if (referer != null) {
					referer = "http://www.nicovideo.jp/my";
					requestHeader.setMessageHeader("Referer",referer);
				}
*/
			}
// ローカルメモリーからflvplayerへのアクセス
			m = CACHECOMMENT_PATTERN2.matcher(uri);
			if (m.matches()) {
				String tail = m.group(3);
				if (tail == null)
					tail ="";
				if (tsParameter == null) {
					Logger.info("ts is not set. Use default.");
					uri = m.group(1)+TS_DEFAULT+tail;
				} else {
					uri = m.group(1)+tsParameter+tail;
				}
				to = m.group(0);
			}
// 自動保存IDつきページからflvplayerへのアクセス
			m = AUTOCACHE_PATTERN.matcher(uri);
			if (m.matches()) {
				String tail = m.group(3);
				if (tail == null)
					tail ="";
				if (tsParameter == null) {
					Logger.info("ts is not set. Use default.");
					uri = m.group(1)+TS_DEFAULT+tail;
				} else {
					uri = m.group(1)+tsParameter+tail;
				}
				to = m.group(0);
			}
// URIを書き戻す
			requestHeader.setURI(uri);
			if (to != null)
				requestHeader.setParameter("nl-to", to);
		}

		return RequestFilter.OK;
	}


/* Cache処理ブロック (fin) */
/****************************************************************************/
// コメントキャッシュの命名規則は
// sm[0-9]+(動画ID)_yymmddhhmmss(受信時刻)[a-z](フラグ用)_動画名(未実装・未定).xml

    private static final Pattern CACHE_COMMENT_FILE_PATTERN =
            Pattern.compile("^([a-z]{2}[0-9]+)_([0-9]+)([a-z])(_.*)?\\.xml$");

// 投コメ判定にデフォルトのソート使ってるから、Normalをzにしてある
// 手抜きバンザイ
	private static final String NORMAL_COMMENT = "z";
	private static final String AUTHOR_COMMENT = "s";
	private static final String NO_COMMENT = "a";

// コメントキャッシュのリスト保存用
    private static ConcurrentHashMap<String, ConcurrentHashMap<String, File>>
		id2NameMap = new ConcurrentHashMap<String, ConcurrentHashMap<String, File>>();

// 取得したコメントの最終コメント番号
    private static ConcurrentHashMap<String, Integer> id2LastRes = 
			new ConcurrentHashMap<String, Integer>();

//    private static ConcurrentHashMap<String, File> cacheNameMap;

// 自動保存時の投コメのキャッシュ用
    private static ConcurrentHashMap<String, byte[]> tmp_watchId2Buf = 
			 new ConcurrentHashMap<String, byte[]>();
// 自動保存用watchId
	private static AtomicInteger watchIdSource = new AtomicInteger();

/*
 * コメントファイルをMapにマッピングする
 */
    private void searchCacheCommentsOnADirectory(File dir, int depth) {
    	// [nl] ディレクトリ一覧を作る
//        dirList.add(getDirName(dir));
        
        File[] cacheFiles = dir.listFiles();
		ArrayList<String> authorList = new ArrayList<String>();
        for (File file : cacheFiles) {
            if (file.isDirectory()) {
                searchCacheCommentsOnADirectory(file, depth + 1);
            } else if (file.isFile()) {
				String filename = file.getName();
        		Matcher m = CACHE_COMMENT_FILE_PATTERN.matcher(filename);
        		if (m.find()) {
					String id = m.group(1);
					String date = m.group(2);
					String flag = m.group(3);
					ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
					if (nameMap == null) {
						nameMap = new ConcurrentHashMap<String, File>();
					}
// まだ一般コメントが見つかってない投稿者コメントのリスト
	               	if (flag.equals(AUTHOR_COMMENT)) {
						if (nameMap.get(date+NORMAL_COMMENT) == null) {
							authorList.add(date+id);
						}
					}
// Mapにはyymmddhhmmss[a-z]で保存
					nameMap.put(date+flag,file);
                	id2NameMap.put(id, nameMap);
                }
            }
        }
// 投稿者コメントのみのものはエラー扱い
		for (String authorName : authorList) {
			ConcurrentHashMap<String, File> nameMap = id2NameMap.get(authorName.substring(12));
			if (nameMap != null) {
				if (nameMap.get(authorName.substring(0,12)+NORMAL_COMMENT) == null) {
					Logger.info("Author-Comment Only.Remove... : "+
							authorName.substring(12)+"_"+
							authorName.substring(0,12));
					removeCommentFile(authorName.substring(12), authorName.substring(0,12), AUTHOR_COMMENT);
				}
			} else {
				Logger.info("cacheComment check error.");
			}
		}
    }


/*
 * 指定されたキャッシュのFileを返す
 */
	private File getCommentFile(String id, String flag, String date) {
		File file = null;
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap != null) {
			file = nameMap.get(date+flag);
		}
		return file;
	}

/*
 * コメントキャッシュのリストをJSONで返す
 */
	private String getCacheCommentDateAsJson(String id) {
		boolean useCommentNum = false;
		if (System.getProperty("useCommentCacheNum", "true").equals("true")) {
			useCommentNum = true;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		String str = "{\"item\":{\"mode\":\""+System.getProperty("autoCacheComment", "false")+"\"";

		int lastRes = getLastResCacheComment(id);
		str += ",\"lastRes\":\""+lastRes+"\"";

		if (nameMap != null && !nameMap.isEmpty()) {
			TreeSet<String> cacheDateTreeSet = new TreeSet<String>(nameMap.keySet());
			str += ",\"member\":[";

			Iterator i = cacheDateTreeSet.iterator();
			while (i.hasNext()) {
				String key = (String)i.next();
				String date = key.substring(0,12);
				String flag = key.substring(12);

				str += "{\"value\":\"?comment_cache="+flag+date+"\",";
				str += "\"text\":\""+
						date.substring(0,2)+"."+date.substring(2,4)+"."+
						date.substring(4,6)+"."+date.substring(6,8)+":"+
						date.substring(8,10)+":"+date.substring(10);
// 取得済みコメント数を入れる
				if (useCommentNum) {
					str += " (" + getCommentNum(id, date) + ")";
				}
				str += "\"},";
// Tree型でソートされてるので、
// 投稿者コメントの次には同じDateの一般コメントが来るはず
// すでに投稿者コメントモードで登録されてるから飛ばす
				if (flag.equals(AUTHOR_COMMENT) && i.hasNext()) {
					i.next();
				}
			}
// 最後の","を取る
			str = str.substring(0,str.length()-1);
			str += "]";
		}
		str += "}}";
		return str;
	}

/*
 * ID,dateのコメントキャッシュ取得数を返す
 */
	private static final int LASTRES_BUF2 = 1024;
	private int getCommentNum(String id, String date) {
// プールされてなければ、ファイルから取得
		int res = 0;
		File file = id2NameMap.get(id).get(date+NORMAL_COMMENT);
		if (file == null) {
			return 0;
		}
		BufferedInputStream in = null;
		byte[] buf = new byte[LASTRES_BUF2];
		String content = "";
		
		//*
// 全部読み込んでコメント数を正確に検索 [2011/01/03 変更点]
		try {
			content = readFile(file);
		} catch (IOException e) {
		} finally {
			CloseUtil.close(in);
		}
		Pattern pattern = Pattern.compile("</chat>");
		Matcher m = pattern.matcher(content);
		while (m.find()) {
			res++;
		}
		/*/
// 先頭部分だけ読み込む
		try {
			int pos = 0;
			in = new BufferedInputStream(
					new FileInputStream(file));
			while (pos < LASTRES_BUF2) {
				int len = in.read(buf, pos, LASTRES_BUF2 - pos);
				if (len == -1) break;
				pos += len;
			}
			content = new String(buf, 0, pos, "utf-8");
		} catch (IOException e) {
		} finally {
			CloseUtil.close(in);
		}

		Pattern pattern = Pattern.compile(" last_res=\"([0-9]+)\" ");
		Matcher m = pattern.matcher(content);
		if (m.find()) {
			res = Integer.decode(m.group(1));
		}
		pattern = Pattern.compile(" no=\"([0-9]+)\"");
		m = pattern.matcher(content);
		if (m.find()) {
			res += 1 - Integer.decode(m.group(1));
		}
		if (res < 1) {
			res = 0;
		}
		//*/
		return res;
	}


/*
 * コメントをサーバから取ってきて保存する
 */
	private Boolean saveComment(
			HttpRequestHeader requestHeader, String parm)
			throws IOException {
// IDは"sm****"かマイメモリーの数字のどちらか
// "&"以降はコメントをどれだけ保存するか
		Pattern pattern = Pattern.compile("(..[0-9]+)([a-z])(?:&([0-9]+))?");
		Matcher m = pattern.matcher(parm);

		String id;
		String flag;
		String commentLength = null;
		if (m.matches()) {
			id = m.group(1);
			flag = m.group(2);
			commentLength = m.group(3);
		} else {
			pattern = Pattern.compile("([0-9]+)&(..[0-9]+)([a-z])");
			m = pattern.matcher(parm);
			if (m.matches()) {
				id = m.group(2);
				flag = m.group(3);
				commentLength = m.group(1);
			} else {
				return false;
			}
		}
// IDから動画情報を得る
/*	----2010/10/12変更点---ここから----
		ConcurrentHashMap<String, String> getMap = getFlvServer(requestHeader, id);
/*/
		boolean isChannel = false;
		String content_tmp = NicoApiUtil.getFlapiContent("getflv?v=".concat(id), requestHeader, null);
		Map<String, String> getMap = NicoApiUtil.query2map(content_tmp);
		if (getMap.containsKey("error")) return false;
		String extra = "";
		isChannel = getMap.containsKey("needs_key");
		if (isChannel && id.matches("\\d+")) {
			content_tmp = NicoApiUtil.getFlapiContent(
					"getthreadkey?thread=".concat(id), requestHeader, null);
			if (content_tmp != null) {
				Map<String, String> map = NicoApiUtil.query2map(content_tmp);
				String threadKey = map.get("threadkey");
				if (threadKey != null) {
					extra += " threadkey=\"" + threadKey + "\"";
					if (map.containsKey("force_184")) {
						extra += " force_184=\"1\"";
					}
				}
			}
		}
//*/	----2010/10/12変更点---ここまで----
		if (getMap == null)
			return false;

/* ここから未使用 [2011/01/03]
// 動画の長さにあわせてコメント数を設定する
		if (commentLength == null) {
			int movieLength = Integer.decode(getMap.get("l"));
			if (movieLength <300) {
				commentLength = "250";
			} else if (movieLength <600) {
				commentLength = "500";
			} else {
				commentLength = "1000";
			}
			
// マイメモリーは1000で読み出すらしい
// 返ってくるのは動画時間に応じた数だけ
			if (id.matches("^[0-9]+$")) {
				commentLength = "1000";
			}
		}
*/
//Logger.info(getMap.toString());

// メッセージサーバへのアクセスに必要なPOST用データを作る
// これはノーマルメッセージ用
/* [2011/01/03]変更点
		String postData =
			"<packet><thread click_revision=\"0\" user_id=\""+getMap.get("user_id")+"\" "+
			"res_from=\"-"+commentLength+"\" version=\"20061206\" "+
			"thread=\""+getMap.get("thread_id")+"\"" + extra + " />";
*/
		
		String postData = "<packet>";
		if (id.matches("^[0-9]+$") && !isChannel) { // マイメモリー
			postData += "<thread scores=\"1\" thread=\""+getMap.get("thread_id")+"\" version=\"20061206\" res_from=\"-1000\" user_id=\""+getMap.get("user_id")+"\"/>";
		} else if (commentLength == null) {
			// 動画長で変動 100コメ / 分
			int movieLength = Integer.decode(getMap.get("l"));
			int movieLength_m = (int) Math.ceil(movieLength / 60.0);
			if (movieLength < 60) {
				commentLength = "100";
			} else if (movieLength < 300) {
				commentLength = "250";
			} else if (movieLength < 600) {
				commentLength = "500";
			} else {
				commentLength = "1000";
			}
			postData += "<thread scores=\"1\" thread=\""+getMap.get("thread_id")+"\" version=\"20090904\" user_id=\""+getMap.get("user_id")+"\"" + extra + "/>"+
				"<thread_leaves scores=\"1\" thread=\""+getMap.get("thread_id")+"\" user_id=\""+getMap.get("user_id")+"\"" + extra + ">"+
				"0-" + movieLength_m + ":100," + commentLength + "</thread_leaves>";
		} else {
			// 取得コメ数を指定
			postData += "<thread scores=\"1\" click_revision=\"0\" user_id=\""+getMap.get("user_id")+"\" "+
				"res_from=\"-"+commentLength+"\" version=\"20061206\" "+
				"thread=\""+getMap.get("thread_id")+"\"" + extra + " />";
		}


// 投稿者コメントがある時
		if (flag.equals(AUTHOR_COMMENT)) {
// 投コメ用POST
			postData +=
				"<thread fork=\"1\" user_id=\""+getMap.get("user_id")+"\" "+
				"res_from=\"-1000\" version=\"20061206\" "+
				"thread=\""+getMap.get("thread_id")+"\"" + extra + " />";

		}
		postData += "</packet>";

		String url = getMap.get("ms");

// コメントを取得する
		byte[] buf = getCommentContent(
				requestHeader, url, postData);
		if (buf == null)
			return false;
		String content = new String(buf, "utf-8");

// getflvの情報から、保存用のタイムスタンプを取得
		// 2006/12/12 is the day NicoNicoDouga was born
		Date niconico_epoch = new Date(Long.decode("1165849200000"));
Logger.debug("cc:saveComment (get(time)):"+getMap.get("time"));
		Date time = new Date(Long.decode(getMap.get("time")));
		if (time.before(niconico_epoch)) {
			time = new Date(Long.decode(getMap.get("time")+"000"));
		}
		String date = new SimpleDateFormat("yyMMddHHmmss").format(time);
Logger.debug("cc:saveComment (date):"+date);

// 自動更新用最終取得コメントの保存
// 同じ処理が散らばってるし、うまくまとめたい
		int lastRes = 0;
		pattern = Pattern.compile(" last_res=\"([0-9]+)\" ");
		m = pattern.matcher(content);
		if (m.find()) {
			lastRes = Integer.decode(m.group(1));
		}

// マイメモリーからなら、xmlから最終コメント日時&IDを取得
// どっかからマイメモリーの保存時刻を取ってこれないか・・・
		if (id.matches("^[0-9]+$")) {
// 最後のコメントが最終コメント・・・のはず
			int lastIndex = content.lastIndexOf(" date=\"");
			if (lastIndex != -1) {
				String subStr = content.substring(lastIndex);
				pattern = Pattern.compile(" date=\"([0-9]+)\"");
				m = pattern.matcher(subStr);
				if (m.find()) {
					date = new SimpleDateFormat("yyMMddHHmmss").format(
							new Date(
								Long.decode(m.group(1)+"000")));
				} else {
					Logger.info("cacheCommentExtension : cannot get Date from xml. Use serverTime...");
				}
			} else {
				Logger.info("cacheCommentExtension : cannot get Date from xml. Use serverTime...");
			}
// 一般コメにはIDが入ってる。投コメには無いので注意
			pattern = Pattern.compile(" id=\"([a-z]{2}[0-9]+)\"");
			m = pattern.matcher(content);
			if (m.find()) {
				id = m.group(1);
			} else {
				Logger.info("cacheCommentExtension : cannot get ID from xml.");
				return false;
			}
// マイメモリーならlastResは0に
			if (!isChannel) {
				lastRes = 0;
			}
		}

// バッファからファイルに保存、キャッシュリストに登録
		if (!saveCommentFile(buf, id, date, NORMAL_COMMENT))
			return false;

/* 投コメは一括取得に変更 -----------------------------

// 投稿者コメントがある時
		if (flag.equals(AUTHOR_COMMENT)) {
// 投コメ用POST
			postData =
				"<thread fork=\"1\" user_id=\""+getMap.get("user_id")+"\" "+
				"res_from=\"-1000\" version=\"20061206\" "+
				"thread=\""+getMap.get("thread_id")+"\" />";

			buf = getCommentContent(
					requestHeader, url, postData);
			if (buf == null) {
// 投コメが取得できなかったら一般コメも削除
				removeCommentFile(id, date, NORMAL_COMMENT);
				return false;
			}
			if (!saveCommentFile(buf, id, date, AUTHOR_COMMENT)) {
// 投コメが保存できなくても一般コメも削除
				removeCommentFile(id, date, NORMAL_COMMENT);
				return false;
			}
		}
// -----------------------------------------------------
*/

		if (lastRes != 0)
			setLastResCacheComment(id, lastRes);
		
		// コメント数を実測
		pattern = Pattern.compile("</chat>");
		m = pattern.matcher(content);
		int len  = 0;
		while (m.find()) {
			len++;
    	}
		Logger.info("Comment save : "+id + " (" + len + "件)");
		return true;
	}

/*
 * /api/getflv から動画情報のGET
 */
	private ConcurrentHashMap<String, String> getFlvServer(
			HttpRequestHeader requestHeader, String id)
			throws IOException {
// ブラウザからGETで来たアクセスのクッキーを使って、
// /api/getflv からメッセージサーバの情報を得る
// いくつかのヘッダデータ(Refererとか)は決め打ちで捏造してる
// がんばるなら、他のアクセスのときに保存しておいて、
// ここのアクセスでは保存された最新のものを使って、
// もし無かったらデフォルトを使うのが正式
		String url = "http://flapi.nicovideo.jp/api/getflv/"+id+"?as3=1";
		URLResource r = new URLResource(url);
// リダイレクトを追跡する
		r.setFollowRedirects(true);

		HttpRequestHeader header = new HttpRequestHeader(HttpHeader.GET+" "+url+" HTTP/1.0\r\n\r\n");
		header.setMessageHeader("Accept", "*/*");
		header.setMessageHeader("Referer", "http://res.nimg.jp/swf/player/nicoplayer.swf?ts=1259739382");
		header.setMessageHeader("x-flash-version", "9,0,124,0");
		header.setMessageHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322)");
		header.setMessageHeader("Connection", "Keep-Alive");
		header.setMessageHeader("Cookie", requestHeader.getMessageHeader("Cookie"));

// 読み込み&ヘッダと本文のパース
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		r.transferTo(null, bout, header, null);

		ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
		HttpResponseHeader responseHeader = new HttpResponseHeader(bin);
		byte[] bcontent = new byte[bin.available()];
		bin.read(bcontent, 0, bin.available());

		if (responseHeader.getStatusCode() != 200) {
			return null;
		}

		String str = new String(bcontent, "ISO-8859-1");
// 受信したデータをマッピングする
		ConcurrentHashMap<String, String> getMap = new ConcurrentHashMap<String, String>();

		Pattern pattern = Pattern.compile("^([^&=]+)=([^&=]+)(?:&(.*))?$");
		while (str != null) {
			Matcher m = pattern.matcher(str);
			if (m.matches()) {
				getMap.put(URLDecoder.decode(m.group(1), "utf-8"),
						URLDecoder.decode(m.group(2), "utf-8"));
				str = m.group(3);
			} else {
				Logger.info("comment Pharse error.");
				return null;
			}
		}
		return getMap;
	}

/*
 * コメントの取得
 */
	private byte[] getCommentContent(
			HttpRequestHeader requestHeader, String url, String postData)
			throws IOException {


		byte[] postBin = postData.getBytes("ISO-8859-1");
		HttpRequestHeader header = new HttpRequestHeader(HttpHeader.POST+" "+url+" HTTP/1.1\r\n\r\n");

		header.setMessageHeader("Accept", "*/*");
		header.setMessageHeader("Accept-Encoding", "gzip");
		header.setMessageHeader("Referer", "http://res.nimg.jp/swf/player/nicoplayer.swf?ts=1259739382");
		header.setMessageHeader("x-flash-version", "9,0,124,0");
		header.setMessageHeader("Content-Type", "application/x-www-form-urlencoded");
		header.setMessageHeader("Content-Length", ""+postBin.length);
		header.setMessageHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 1.1.4322)");
		header.setMessageHeader("Connection", "Keep-Alive");
		header.setMessageHeader("Pragma", "no-cache");
		header.setMessageHeader("Cookie", requestHeader.getMessageHeader("Cookie"));

		return postServer(header, postBin);
	}

/*
 * サーバーへのPOST
 */

	private byte[] postServer(
			HttpRequestHeader requestHeader, byte[] postBin)
			throws IOException {
// 解凍できないEncodingは削除
		String acceptEncoding = requestHeader.getMessageHeader("Accept-Encoding");
		if (acceptEncoding != null) {
			acceptEncoding = acceptEncoding.toLowerCase().replaceAll("( *,)?(bzip2|sdch)", "");
			acceptEncoding = acceptEncoding.replaceFirst("^ *," ,"");
			requestHeader.setMessageHeader("Accept-Encoding", acceptEncoding);
		}

		URLResource r = new URLResource(requestHeader.getURI());
		ByteArrayInputStream bin = new ByteArrayInputStream(postBin);
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		r.transferTo(bin, bout, requestHeader, null);

		bin = new ByteArrayInputStream(bout.toByteArray());
		HttpResponseHeader responseHeader = new HttpResponseHeader(bin);
		byte[] bcontent = new byte[bin.available()];
		bin.read(bcontent, 0, bin.available());

		if (responseHeader.getStatusCode() != 200) {
			return null;
		}

// 圧縮されていたら解凍
		String contentEncoding = responseHeader.getMessageHeader("Content-Encoding");
		if (contentEncoding != null) {
			contentEncoding = contentEncoding.toLowerCase();
			if (contentEncoding.equals("gzip") ||
					contentEncoding.equals("deflate")) {
				bcontent = decodeGzip(bcontent, contentEncoding);
			} else if (!contentEncoding.equals("identity")) {
				Logger.info("Unknown compress format. save comment failed.");
				return null;
			}
		}
		return bcontent;
	}

/*
 * バッファからファイルに保存、キャッシュリストに登録
 */
	private boolean saveCommentFile(
			byte[] buf, String id, String date, String flag)
				throws IOException {
// ファイルネームを作る
		String fileName = cacheCommentDir+"/"+id+"_"+date+flag+".xml";

		File file = new File(fileName);
		BufferedOutputStream bos =
				new BufferedOutputStream(new FileOutputStream(file));
		try {
			bos.write(buf);
			bos.flush();
		} catch (IOException e) {
			Logger.info("comment save error.");
			return false;
		} finally {
			CloseUtil.close(bos);
		}

// リストに登録
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap == null) {
			nameMap = new ConcurrentHashMap<String, File>();
		}
		nameMap.put(date+flag, file);
		id2NameMap.put(id, nameMap);

		return true;
	}

/*
 * IDのコメントキャッシュが存在していたらtrue
 */
	private boolean existComment(String id) {
		if (id == null) {
			return false;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
// idがキーにあっても、削除されて中身がない時もあるかも
		if (nameMap == null) {
			return false;
		}
		if (nameMap.size() == 0) {
			return false;
		}
		return true;
}

/*
 * ID、DATEのコメントキャッシュが存在していたらtrue
 */
	private boolean existComment(String id, String date) {
		if (id == null) {
			return false;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
// idがキーにあっても、削除されて中身がない時もあるかも
		if (nameMap == null) {
			return false;
		}
		if (nameMap.get(date+NORMAL_COMMENT) == null) {
			return false;
		}
		return true;
}

/*
 * 圧縮を解凍する
 */
	private static final int GZIP_BUF_SIZE = 1024*10;
	private byte[] decodeGzip(byte[] bcontent, String format) {
		BufferedInputStream in = null;
 		ByteArrayOutputStream out = new ByteArrayOutputStream();
		int len;
		byte[] buf = new byte[GZIP_BUF_SIZE];

		try {
			if (format.equals("gzip")) {
				in = new BufferedInputStream(
						new GZIPInputStream(
							new ByteArrayInputStream(bcontent)));
			} else if (format.equals("deflate")) {
				in = new BufferedInputStream(
						new InflaterInputStream(
							new ByteArrayInputStream(bcontent)));
			}
			while ( (len = in.read(buf, 0, GZIP_BUF_SIZE)) != -1) {
				out.write(buf, 0, len);
			}
		} catch (IOException e) {
			return null;
		} finally {
			CloseUtil.close(in);
		}
		return out.toByteArray();
	}

/*
 * ブラウザが読み込んでいるコメントをキャッシュする
 * 投コメは一時バッファに入れて、後でペアにする
 */
	private void storeComment(byte[] buf, String watchId, String postString)
				throws IOException {
		String id = null;
		String content = new String(buf, "utf-8");
		Pattern pattern = Pattern.compile(" id=\"([a-z]{2}[0-9]+)\"");
		Matcher m = pattern.matcher(content);
		if (m.find()) {
			id = m.group(1);
		}

// watchページのIDからマイメモリーかどうか判定
// nullでなければマイメモリーになる
		pattern = Pattern.compile("[0-9]+t([0-9]+)");
		m = pattern.matcher(watchId);
		String pageId = null;
		if (m.find()) {
			pageId = m.group(1);
		}
// 投コメなら一時バッファに保存
		if (id == null) {
			tmp_watchId2Buf.put(watchId, buf);
			return;
		}
// 一般コメ
		boolean keepComment = false;
		String configAutoCacheComment = System.getProperty("autoCacheComment", "false");
		int lastRes = 0;
		pattern = Pattern.compile(" last_res=\"([0-9]+)\" ");
		m = pattern.matcher(content);
		if (m.find()) {
			lastRes = Integer.decode(m.group(1));
		}

// マイメモリーは保存処理にまわす。
// マイメモリーのlastResはキャッシュしないので0にする
// マイメモリーは常に保存処理するので、これより新しいコメントが
// 保存されている&lastResがまだキャッシュされていないと、
// マイメモリーのlastResが最新と勘違いするから
		if (pageId != null) {
			lastRes = 0;
			keepComment = true;
// ファイルがない時も保存
		} else if (!existComment(id)) {
			keepComment = true;
// oneモードじゃなければ、新しいレスの数を調べる
		} else if (!configAutoCacheComment.equals("one")) {
			int oldRes = 0;
			int existRes = getLastResCacheComment(id);
			int refreshThreshold = 0;

			if (configAutoCacheComment.matches("[0-9]+")) {
				refreshThreshold = Integer.decode(configAutoCacheComment);
			}
			pattern = Pattern.compile(" no=\"([0-9]+)\" ");
			m = pattern.matcher(content);
			if (m.find()) {
				oldRes = Integer.decode(m.group(1));
			}
			if (lastRes != 0 && oldRes != 0) {
				int maxRes = 0;
				int newRes = lastRes - existRes;
				pattern = Pattern.compile(">0-([0-9]+):[0-9]+,([0-9]+)</thread_leaves>");
				m = pattern.matcher(postString);
				if (m.find()) {
			//		maxRes = Integer.decode(m.group(1)) * 100 + Integer.decode(m.group(2));	// とりあえず、100コメ/分固定らしい
					maxRes = Integer.decode(m.group(1)) * 100;	// 後ろの旧コメント数を足すと全然更新されないから上書き
				} else {
					pattern = Pattern.compile(">0-([0-9]+):[0-9]+</thread_leaves>");
					m = pattern.matcher(postString);
					if (m.find()) {
						maxRes = Integer.decode(m.group(1)) * 100;
					} else {
						pattern = Pattern.compile(" res_from=\"-([0-9]+)\" ");
						m = pattern.matcher(postString);
						if (m.find()) {
							maxRes = Integer.decode(m.group(1));
						}
					}
				}
//				Logger.info("[debug] " + id + " : maxRes = " + maxRes + " : newRes = " + newRes);
// 結構いい加減な判定。総コメント数が少ない時には誤爆する
				if (maxRes == 0){
					maxRes = lastRes - oldRes +1;
					if (maxRes > 1000) {
						// 仕様変更で1000コメ以上の動画が発生(10分以上で)
					} else if (lastRes > 500 && maxRes > 500) {
						maxRes = 1000;
					} else if (lastRes > 250 && maxRes > 250) {
						maxRes = 500;
					} else if (lastRes > 100 && maxRes > 100) {
						maxRes = 250;
					} else {
						maxRes = 100;
					}
				}
				if (maxRes * refreshThreshold / 100 < newRes) {
					keepComment = true;
				}
			}
		}

// 上で保存すると判定されていれば
		if (keepComment) {
// xmlからアクセス時のサーバ時間を取得
			String time;
			pattern = Pattern.compile(" server_time=\"([0-9]+)\"");
			m = pattern.matcher(content);
			if (m.find()) {
				time = m.group(1);
			} else {
// 時間が取得できなければエラー
				return;
			}
			String date = new SimpleDateFormat("yyMMddHHmmss").format(
					new Date(Long.decode(time+"000")));
// マイメモリーからなら、xmlから最終コメント日時を取得
			if (pageId != null) {
// 最後のコメントが最終コメント・・・のはず
				if (content.lastIndexOf(" date=\"") == -1) {
					return;
				}
				String subStr = content.substring(content.lastIndexOf(" date=\""));
				pattern = Pattern.compile(" date=\"([0-9]+)\"");
				m = pattern.matcher(subStr);
				if (m.find()) {
					date = new SimpleDateFormat("yyMMddHHmmss").format(
							new Date(
								Long.decode(m.group(1)+"000")));
				} else {
					Logger.info("cacheCommentExtension : cannot get Date from xml. Use serverTime...");
				}
			}

// すでに同じ日付で保存されていたらスルー
			if (!existComment(id, date)) {
	 			if (!saveCommentFile(buf, id, date, NORMAL_COMMENT)) {
					return;
				}
// 対応する投コメがあるかどうか
				byte[] tmp_buf = tmp_watchId2Buf.get(watchId);
				if (tmp_buf != null) {
// 対応する投コメの保存に失敗したら一般コメも削除
					if (!saveCommentFile(tmp_buf, id, date, AUTHOR_COMMENT)) {
						removeCommentFile(id, date, NORMAL_COMMENT);
						return;
					}
				}
// lastResを保存。0の時は保存しない
				if (lastRes != 0)
					setLastResCacheComment(id, lastRes);
				
				pattern = Pattern.compile("</chat>");
				m = pattern.matcher(content);
				int commentLength = 0;
				while (m.find()) {
					commentLength++;
        		}
        		Logger.info("autoCacheComment : "+id + " (" + commentLength + "件)");
			}
		}
// 一般コメの処理が終わったら、対応する投コメの一時キャッシュを削除
// 上でエラーだったら残っちゃうけど、そのうち消えるからいいか
		removeTempComment(watchId);
	}

/*
 * IDの動画で保存されたコメントの、最後のコメント番号を設定する
 */
	private void setLastResCacheComment(String id, int lastRes) {
		Integer res = id2LastRes.get(id);
		if (res == null || res.intValue() < lastRes) {
			id2LastRes.put(id, lastRes);
		}
	}

/*
 * IDの動画で保存されたコメントの、最終コメント番号を取得する
 */
	private static final int LASTRES_BUF = 256;
	private int getLastResCacheComment(String id) {
		Integer res = id2LastRes.get(id);
// プールされてなければ、ファイルから取得
		if (res == null) {
			res = 0;
			TreeSet<String> dateTree = getCacheCommentDateTreeSet(id);
			String date;
			if (dateTree != null && !dateTree.isEmpty()) {
				date = dateTree.last();
				File file = id2NameMap.get(id).get(date);
				BufferedInputStream in = null;
				byte[] buf = new byte[LASTRES_BUF];
				String content = "";
// 先頭部分だけ読み込む
				try {
					int pos = 0;
					in = new BufferedInputStream(
							new FileInputStream(file));
					while (pos < LASTRES_BUF) {
						int len = in.read(buf, pos, LASTRES_BUF-pos);
						if (len == -1) break;
						pos += len;
					}
					content = new String(buf, 0, pos, "utf-8");
				} catch (IOException e) {
 				} finally {
					CloseUtil.close(in);
				}

				Pattern pattern = Pattern.compile(" last_res=\"([0-9]+)\" ");
				Matcher m = pattern.matcher(content);
				if (m.find()) {
					res = Integer.decode(m.group(1));
				} else {
		//			Logger.info("[debug] cc:LASTRES_BUF is too small");
				}
				pattern = Pattern.compile(" res_offset=\"([0-9]+)\" ");
				m = pattern.matcher(content);
				if (m.find()) {
					res += Integer.decode(m.group(1));
				}
			}
// 取得したらキャッシュへ。もし読み込めなかったら0
			setLastResCacheComment(id, res);
		}
		return res;
	}

/*
 * IDにあるコメントの日付のTreeSetを返す。ソートされてる
 * 後ろにflagがついてる
 */
	private TreeSet<String> getCacheCommentDateTreeSet(String id) {
		TreeSet<String> cacheDateTreeSet = null;
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap != null) {
			cacheDateTreeSet = new TreeSet<String>(nameMap.keySet());
		}
		return cacheDateTreeSet;
	}

/*
 * 指定したwatchIdの一時キャッシュを削除する
 */
	private void removeTempComment(String watchId) {
		tmp_watchId2Buf.remove(watchId);
// 期限切れの投コメ一時キャッシュを削除する
// とりあえず120秒にしてみる
		Pattern pattern = Pattern.compile("^[0-9]+t([0-9]+)$");
		Matcher m;
		for (String idTemp : tmp_watchId2Buf.keySet()) {
			m = pattern.matcher(idTemp);
			if (m.matches()) {
				long time = Long.decode(m.group(1));
				if (time < new Date().getTime() -120*1000) {
					tmp_watchId2Buf.remove(idTemp);
				}
			} else {
// 時間の取得できない妙な一時キャッシュは削除
				tmp_watchId2Buf.remove(idTemp);
			}
		}
	}

/*
 * AtomicなIDを返す
 */
	private String getWatchIdComment() {
		return watchIdSource.getAndIncrement()+"t";
	}

/*
 * コメントファイルの削除
 */
// ID,時間、種別指定
	private void removeCommentFile(
				String id, String date, String flag) {
		if (id == null) {
			return;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap == null) {
			return;
		}
		File file = nameMap.get(date+flag);
		if (file != null) {
			file.delete();
		}
		nameMap.remove(date+flag);
		id2NameMap.put(id, nameMap);
	}

// IDと時間指定
	private void removeCommentFile(String id, String date) {
		if (id == null) {
			return;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap == null) {
			return;
		}
		File file = nameMap.get(date+NORMAL_COMMENT);
		if (file != null) {
			file.delete();
			nameMap.remove(date+NORMAL_COMMENT);
		}
		file = nameMap.get(date+AUTHOR_COMMENT);
		if (file != null) {
			file.delete();
			nameMap.remove(date+AUTHOR_COMMENT);
		}
		if (nameMap.isEmpty()) {
			id2NameMap.remove(id);
		} else {
			id2NameMap.put(id, nameMap);
		}
		id2LastRes.remove(id);
	}

// IDのみ指定で削除
	private void removeCommentFile(String id) {
		if (id == null) {
			return;
		}
		ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
		if (nameMap == null) {
			return;
		}
		File file;
		for (Iterator<File> j = nameMap.values().iterator(); j.hasNext();) {
			file = j.next();
			if (file != null) {
				file.delete();
			}
		}
		id2NameMap.remove(id);
		id2LastRes.remove(id);
	}

// 動画キャッシュのないコメントを削除
	private void removeCommentExistMovie() {
		String id;
		for (Iterator<String> j = id2NameMap.keySet().iterator(); j.hasNext();) {
			id = j.next();
			if ( Cache.getType(id) == null) {
				removeCommentFile(id);
			}
		}
	}

// コメント数が引数以下のコメントファイルを削除
	private void removeShortCommentFile(int commentNum) {
		String id;
		for (Iterator<String> i = id2NameMap.keySet().iterator(); i.hasNext();) {
			id = i.next();
			ConcurrentHashMap<String, File> nameMap = id2NameMap.get(id);
			String dates;
			for (Iterator<String> j = nameMap.keySet().iterator(); j.hasNext();) {
				dates = j.next();
				if (dates.endsWith(NORMAL_COMMENT)) {
// コメント数は面倒なんで最終取得コメ番号から類推する
					int res = 0;
					File file = nameMap.get(dates);
					BufferedInputStream in = null;
					byte[] buf = new byte[LASTRES_BUF];
					String content = "";
// 先頭部分だけ読み込む
					try {
						int pos = 0;
						in = new BufferedInputStream(
								new FileInputStream(file));
						while (pos < LASTRES_BUF) {
							int len = in.read(buf, pos, LASTRES_BUF-pos);
							if (len == -1) break;
							pos += len;
						}
						content = new String(buf, 0, pos, "utf-8");
					} catch (IOException e) {
 					} finally {
						CloseUtil.close(in);
					}

					Pattern pattern = Pattern.compile(" last_res=\"([0-9]+)\" ");
					Matcher m = pattern.matcher(content);
					if (m.find()) {
						res += Integer.decode(m.group(1));
					}
					pattern = Pattern.compile(" res_offset=\"([0-9]+)\" ");
					m = pattern.matcher(content);
					if (m.find()) {
						res += Integer.decode(m.group(1));
					}
// 少なかったら削除
					if (res < commentNum) {
						removeCommentFile(id, dates.substring(0, dates.length()-1));
					}

				}
			}
		}
	}
	
	private String addTS(String fileName){
		Matcher m = Pattern.compile("^(?:http://[^/]*\\.nicovideo\\.jp/|/)?(local/[^?]+)").matcher(fileName);
		if (m.find()) {
			// ファイルが存在する場合のみパラメータを付加
			long lastModified = new File(m.group(1)).lastModified();
			if (lastModified > 0L) {
				fileName += "?" + (lastModified / 1000L);
			}
		}
		return fileName;
	}
	
	private String addTS2(String fileName){
		Matcher m = Pattern.compile("^(.*\"(?:http://[^/]*\\.nicovideo\\.jp/|/)?(local/[^?\"]+)(?:[?][^\"]*)?)(\".*$)").matcher(fileName);
		if (m.find()) {
			// ファイルが存在する場合のみパラメータを付加
			long lastModified = new File(m.group(2)).lastModified();
			if (lastModified > 0L) {
				fileName = m.group(1) + "?" + (lastModified / 1000L) + m.group(3);
			}
		}
		return fileName;
	}
}
/* end */

